@@ -8,17 +8,13 @@
 
 /*--------------------------------------------------------------------------*/
 
-/* Make sure the information strings are referenced.  */
-#define REQUIRE(x) (&x[0] != &require)
-
-int main()
+int main(int argc, char* argv[])
 {
-  const char require = 0;
-  return
-    (
-      REQUIRE(info_sizeof_dptr)
+  int require = 0;
+  require += info_sizeof_dptr[argc];
 #if defined(ABI_ID)
-      && REQUIRE(info_abi)
+  require += info_abi[argc];
 #endif
-      );
+  (void)argv;
+  return require;
 }
