@@ -12,12 +12,20 @@
 
 #include "cmExportSet.h"
 #include "cmTargetExport.h"
+#include "cmAlgorithms.h"
+#include "cmLocalGenerator.h"
 
 cmExportSet::~cmExportSet()
 {
-  for(unsigned int i = 0; i < this->TargetExports.size(); ++ i)
+  cmDeleteAll(this->TargetExports);
+}
+
+void cmExportSet::Compute(cmLocalGenerator* lg)
+{
+  for (std::vector<cmTargetExport*>::iterator it = this->TargetExports.begin();
+       it != this->TargetExports.end(); ++it)
     {
-    delete this->TargetExports[i];
+    (*it)->Target = lg->FindGeneratorTargetToUse((*it)->TargetName);
     }
 }
 
