@@ -12,6 +12,7 @@
 
 #include "cmExportSetMap.h"
 #include "cmExportSet.h"
+#include "cmAlgorithms.h"
 
 cmExportSet* cmExportSetMap::operator[](const std::string &name)
 {
@@ -25,12 +26,7 @@ cmExportSet* cmExportSetMap::operator[](const std::string &name)
 
 void cmExportSetMap::clear()
 {
-  for(std::map<std::string, cmExportSet*>::iterator it = this->begin();
-      it != this->end();
-      ++ it)
-    {
-    delete it->second;
-    }
+  cmDeleteAll(*this);
   this->derived::clear();
 }
 
