@@ -10,7 +10,6 @@
   See the License for more information.
 ============================================================================*/
 #include "cmFindPathCommand.h"
-#include "cmCacheManager.h"
 
 #include <cmsys/Glob.hxx>
 
@@ -41,7 +40,7 @@ bool cmFindPathCommand
         this->VariableName, "",
         this->VariableDocumentation.c_str(),
         (this->IncludeFileInPath ?
-         cmCacheManager::FILEPATH :cmCacheManager::PATH)
+         cmState::FILEPATH :cmState::PATH)
         );
       }
     return true;
@@ -54,7 +53,7 @@ bool cmFindPathCommand
       (this->VariableName, result.c_str(),
        this->VariableDocumentation.c_str(),
        (this->IncludeFileInPath) ?
-       cmCacheManager::FILEPATH :cmCacheManager::PATH);
+       cmState::FILEPATH :cmState::PATH);
     return true;
     }
   this->Makefile->AddCacheDefinition
@@ -62,7 +61,7 @@ bool cmFindPathCommand
      (this->VariableName + "-NOTFOUND").c_str(),
      this->VariableDocumentation.c_str(),
      (this->IncludeFileInPath) ?
-     cmCacheManager::FILEPATH :cmCacheManager::PATH);
+     cmState::FILEPATH :cmState::PATH);
   return true;
 }
 
