@@ -1,21 +1,29 @@
-/*=========================================================================
+/*============================================================================
+  CMake - Cross Platform Makefile Generator
+  Copyright 2000-2009 Kitware, Inc., Insight Software Consortium
 
-  Program:   CMake - Cross-Platform Makefile Generator
-  Module:    $RCSfile: cmForEachCommand.cxx,v $
-  Language:  C++
-  Date:      $Date: 2009-02-04 16:44:17 $
-  Version:   $Revision: 1.27.2.1 $
+  Distributed under the OSI-approved BSD License (the "License");
+  see accompanying file Copyright.txt for details.
 
-  Copyright (c) 2002 Kitware, Inc., Insight Consortium.  All rights reserved.
-  See Copyright.txt or http://www.cmake.org/HTML/Copyright.html for details.
-
-     This software is distributed WITHOUT ANY WARRANTY; without even 
-     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
-     PURPOSE.  See the above copyright notices for more information.
-
-=========================================================================*/
+  This software is distributed WITHOUT ANY WARRANTY; without even the
+  implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+  See the License for more information.
+============================================================================*/
 #include "cmForEachCommand.h"
 
+#include <cmsys/auto_ptr.hxx>
+
+cmForEachFunctionBlocker::cmForEachFunctionBlocker(cmMakefile* mf):
+  Makefile(mf), Depth(0)
+{
+  this->Makefile->PushLoopBlock();
+}
+
+cmForEachFunctionBlocker::~cmForEachFunctionBlocker()
+{
+  this->Makefile->PopLoopBlock();
+}
+
 bool cmForEachFunctionBlocker::
 IsFunctionBlocked(const cmListFileFunction& lff, cmMakefile &mf,
                   cmExecutionStatus &inStatus)
@@ -28,7 +36,7 @@ IsFunctionBlocked(const cmListFileFunction& lff, cmMakefile &mf,
   else if (!cmSystemTools::Strucmp(lff.Name.c_str(),"endforeach"))
     {
     // if this is the endofreach for this statement
-    if (!this->Depth) 
+    if (!this->Depth)
       {
       // Remove the function blocker for this scope or bail.
       cmsys::auto_ptr<cmFunctionBlocker>
@@ -38,19 +46,17 @@ IsFunctionBlocked(const cmListFileFunction& lff, cmMakefile &mf,
       // at end of for each execute recorded commands
       // store the old value
       std::string oldDef;
-      if (mf.GetDefinition(this->Args[0].c_str()))
+      if (mf.GetDefinition(this->Args[0]))
         {
-        oldDef = mf.GetDefinition(this->Args[0].c_str());
+        oldDef = mf.GetDefinition(this->Args[0]);
         }
       std::vector<std::string>::const_iterator j = this->Args.begin();
       ++j;
 
-      std::string tmps;
-      cmListFileArgument arg;
       for( ; j != this->Args.end(); ++j)
-        {   
+        {
         // set the variable to the loop value
-        mf.AddDefinition(this->Args[0].c_str(),j->c_str());
+        mf.AddDefinition(this->Args[0],j->c_str());
         // Invoke all the functions that were collected in the block.
         cmExecutionStatus status;
         for(unsigned int c = 0; c < this->Functions.size(); ++c)
@@ -61,19 +67,28 @@ IsFunctionBlocked(const cmListFileFunction& lff, cmMakefile &mf,
             {
             inStatus.SetReturnInvoked(true);
             // restore the variable to its prior value
-            mf.AddDefinition(this->Args[0].c_str(),oldDef.c_str());
+            mf.AddDefinition(this->Args[0],oldDef.c_str());
             return true;
             }
           if (status.GetBreakInvoked())
             {
             // restore the variable to its prior value
-            mf.AddDefinition(this->Args[0].c_str(),oldDef.c_str());
+            mf.AddDefinition(this->Args[0],oldDef.c_str());
+            return true;
+            }
+          if (status.GetContinueInvoked())
+            {
+            break;
+            }
+          if(cmSystemTools::GetFatalErrorOccured() )
+            {
             return true;
             }
           }
         }
+
       // restore the variable to its prior value
-      mf.AddDefinition(this->Args[0].c_str(),oldDef.c_str());
+      mf.AddDefinition(this->Args[0],oldDef.c_str());
       return true;
       }
     else
@@ -82,10 +97,10 @@ IsFunctionBlocked(const cmListFileFunction& lff, cmMakefile &mf,
       this->Depth--;
       }
     }
-  
+
   // record the command
   this->Functions.push_back(lff);
-  
+
   // always return true
   return true;
 }
@@ -116,9 +131,13 @@ bool cmForEachCommand
     this->SetError("called with incorrect number of arguments");
     return false;
     }
-  
+  if(args.size() > 1 && args[1] == "IN")
+    {
+    return this->HandleInMode(args);
+    }
+
   // create a function blocker
-  cmForEachFunctionBlocker *f = new cmForEachFunctionBlocker();
+  cmForEachFunctionBlocker *f = new cmForEachFunctionBlocker(this->Makefile);
   if ( args.size() > 1 )
     {
     if ( args[1] == "RANGE" )
@@ -152,16 +171,16 @@ bool cmForEachCommand
           step = 1;
           }
         }
-      if ( 
+      if (
         (start > stop && step > 0) ||
         (start < stop && step < 0) ||
         step == 0
         )
         {
-        cmOStringStream str;
+        std::ostringstream str;
         str << "called with incorrect range specification: start ";
         str << start << ", stop " << stop << ", step " << step;
-        this->SetError(str.str().c_str());
+        this->SetError(str.str());
         return false;
         }
       std::vector<std::string> range;
@@ -193,7 +212,51 @@ bool cmForEachCommand
     f->Args = args;
     }
   this->Makefile->AddFunctionBlocker(f);
-  
+
   return true;
 }
 
+//----------------------------------------------------------------------------
+bool cmForEachCommand::HandleInMode(std::vector<std::string> const& args)
+{
+  cmsys::auto_ptr<cmForEachFunctionBlocker>
+    f(new cmForEachFunctionBlocker(this->Makefile));
+  f->Args.push_back(args[0]);
+
+  enum Doing { DoingNone, DoingLists, DoingItems };
+  Doing doing = DoingNone;
+  for(unsigned int i=2; i < args.size(); ++i)
+    {
+    if(doing == DoingItems)
+      {
+      f->Args.push_back(args[i]);
+      }
+    else if(args[i] == "LISTS")
+      {
+      doing = DoingLists;
+      }
+    else if(args[i] == "ITEMS")
+      {
+      doing = DoingItems;
+      }
+    else if(doing == DoingLists)
+      {
+      const char* value = this->Makefile->GetDefinition(args[i]);
+      if(value && *value)
+        {
+        cmSystemTools::ExpandListArgument(value, f->Args, true);
+        }
+      }
+    else
+      {
+      std::ostringstream e;
+      e << "Unknown argument:\n" << "  " << args[i] << "\n";
+      this->Makefile->IssueMessage(cmake::FATAL_ERROR, e.str());
+      return true;
+      }
+    }
+
+  this->Makefile->AddFunctionBlocker(f.release()); // TODO: pass auto_ptr
+
+  return true;
+}
