@@ -46,7 +46,7 @@ public:
   /**
    * The name of the command as specified in CMakeList.txt.
    */
-  virtual const char* GetName() const { return "get_filename_component";}
+  virtual std::string GetName() const { return "get_filename_component";}
 
   cmTypeMacro(cmGetFilenameComponentCommand, cmCommand);
 };
