@@ -13,7 +13,8 @@
 #define cmOutputRequiredFilesCommand_h
 
 #include "cmCommand.h"
-#include "cmMakeDepend.h"
+
+class cmDependInformation;
 
 class cmOutputRequiredFilesCommand : public cmCommand
 {
@@ -23,7 +24,6 @@ public:
   virtual bool InitialPass(std::vector<std::string> const& args,
                            cmExecutionStatus &status);
   virtual std::string GetName() const { return "output_required_files";}
-  virtual bool IsDiscouraged() const { return true; }
 
   void ListDependencies(cmDependInformation const *info,
                         FILE *fout,
