@@ -177,7 +177,8 @@ static cmVS7FlagTable cmVS11LinkFlagTable[] =
   {"UACUIAccess", "uiAccess='false'", "", "false", 0},
   {"UACUIAccess", "uiAccess='true'", "", "true", 0},
   {"ManifestEmbed", "manifest:embed", "", "true", 0},
-  {"GenerateDebugInformation", "DEBUG", "", "true", 0},
+  {"GenerateDebugInformation", "DEBUG", "", "true",
+   cmVS7FlagTable::CaseInsensitive},
   {"MapExports", "MAPINFO:EXPORTS", "", "true", 0},
   {"AssemblyDebug", "ASSEMBLYDEBUG:DISABLE", "", "false", 0},
   {"AssemblyDebug", "ASSEMBLYDEBUG", "", "true", 0},
